//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: ExN03EventAction.cc,v 1.1 2005/05/18 14:54:22 ribon Exp $
// GEANT4 tag $Name:  $
//
// 

#include "ExN03EventAction.hh"
#include "ExN03EventActionMessenger.hh"

#include "G4Event.hh"
#include "G4TrajectoryContainer.hh"
#include "G4VTrajectory.hh"
#include "G4VVisManager.hh"
#include "G4UnitsTable.hh"
#include "G4SDManager.hh"
#include "Randomize.hh"
#include <iomanip>
#include <fstream>
#include "MySensitiveTracker.hh"
#include "MyTrackerHit.hh"

//***LOOKHERE***
#include "MyAnalysis.hh"
#include "G4EventManager.hh"
#include "G4HCofThisEvent.hh"
#include "G4VHitsCollection.hh"
//**************


ExN03EventAction::ExN03EventAction()
:drawFlag("all"),printModulo(1),eventMessenger(0)
{
    G4String colName;
    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    trackerleftCollID = SDman->GetCollectionID(colName="blmleft/driftChamberColl");
    trackerrightCollID = SDman->GetCollectionID(colName="blmright/driftChamberColl");

  eventMessenger = new ExN03EventActionMessenger(this);


}


ExN03EventAction::~ExN03EventAction()
{
  delete eventMessenger;
}


void ExN03EventAction::BeginOfEventAction(const G4Event* evt)
{  
 G4int evtNb = evt->GetEventID();
 if (evtNb%printModulo == 0) { 
   G4cout << "\n---> Begin of event: " << evtNb << G4endl;
   HepRandom::showEngineStatus();
 }
 
}


void ExN03EventAction::EndOfEventAction(const G4Event* evt)
{
  //***LOOKHERE***
      MyAnalysis* analysis = MyAnalysis::getInstance();

  // Get the IDs of the collections of tracker hits. 

  // Extract the hits.
  G4HCofThisEvent* HCE = evt->GetHCofThisEvent();
  MyTrackerHitsCollection* BHCleft = 0;
  MyTrackerHitsCollection* BHCright = 0;

  G4double energyDeposited = 0.0;
  G4double particuleKinEnergy = 0.0; 
  G4ThreeVector pos = 0 ; 
  G4String type = "nul";
  
  if ( HCE ) 
     {
    BHCleft = (MyTrackerHitsCollection*)(HCE->GetHC(trackerleftCollID));
    BHCright = (MyTrackerHitsCollection*)(HCE->GetHC(trackerrightCollID));
      }


  // Fill some histograms

  if (BHCleft )
    { int n_hit = BHCleft->entries();
        G4cout << "     " << n_hit
        << " hits are stored in BLM left" << G4endl;
  // Fill the tuple

        for(int i2=0;i2<n_hit;i2++)
           { 
            energyDeposited= (*BHCleft)[i2]->GetEdep();
            pos=(*BHCleft)[i2]->GetPos();
            type = (*BHCleft)[i2]->GetParticuleName();
            particuleKinEnergy=  (*BHCleft)[i2]-> GetParticuleKinEn();   
            analysis->fill(type,energyDeposited,pos,particuleKinEnergy);
	         }
       
    }
  if (BHCright )
    { int n_hit = BHCright->entries();
        G4cout << "     " << n_hit
        << " hits are stored in BLM right" << G4endl;
         for(int i2=0;i2<n_hit;i2++)
           {             
            energyDeposited= (*BHCright)[i2]->GetEdep();
            pos=(*BHCright)[i2]->GetPos();
            type = (*BHCright)[i2]->GetParticuleName();
            particuleKinEnergy=  (*BHCright)[i2]-> GetParticuleKinEn();   
            analysis->fill(type,energyDeposited,pos,particuleKinEnergy);
	         }
    }
  analysis->tree->AutoSave("SaveSelf");
    


  //**************

 G4int evtNb = evt->GetEventID();
  if (evtNb%printModulo == 0) 
     {
     G4cout << "---> End of event: " << evtNb << G4endl;	
     }


  
  // extract the trajectories and draw them

  // You can get a default drawing without this code by using, e.g.,
  // /vis/scene/add/trajectories 1000
  // The code here adds sophistication under control of drawFlag.

  // See comments in G4VTrajectory::DrawTrajectory for the
  // interpretation of the argument, 1000.
  
  G4VVisManager* pVisManager = G4VVisManager::GetConcreteInstance();
  if (pVisManager)
    {
     G4TrajectoryContainer* trajectoryContainer = evt->GetTrajectoryContainer();
     G4int n_trajectories = 0;
     if (trajectoryContainer) n_trajectories = trajectoryContainer->entries();

     for (G4int i=0; i<n_trajectories; i++) 
        { G4VTrajectory* trj = ((*(evt->GetTrajectoryContainer()))[i]);
          if (drawFlag == "all") pVisManager->Draw(*trj,1000);
          else if ((drawFlag == "charged")&&(trj->GetCharge() != 0.))
                                  pVisManager->Draw(*trj,1000);
          else if ((drawFlag == "neutral")&&(trj->GetCharge() == 0.))
                                  pVisManager->Draw(*trj,1000);
        }
    }
}  

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
